#ifndef CoCoA_EFGModule_H
#define CoCoA_EFGModule_H

//   Copyright (c)  2005  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License (version 2)
//   as published by the Free Software Foundation.  A copy of the full
//   licence may be found in the file COPYING in this directory.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoA; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


// Header file for the classes EFGModule and EFGModuleBase
// EFG = Explicitly Finitely Generated (i.e. submodules and quotients of
// free modules).


#include "CoCoA/error.H"
#include "CoCoA/ring.H"
#include "CoCoA/module.H"

#include <vector>
//using std::vector;
#include <cstddef>
//using std::size_t;


namespace CoCoA
{

  class FreeModule; // forward declaration (just for myAmbientFreeModule)

  class EFGModuleBase; // forward declaration
  class EFGModule: public module
  {
  public:
    // Default copy ctor is OK.
    explicit EFGModule(const EFGModuleBase* MPtr);   //??? public???
    explicit EFGModule(const module& M); // specializing copy ctor
//???    module& operator=(const module& rhs);
//??? default dtor is fine???    virtual ~EFGModule();

    const EFGModuleBase* myModulePtr() const;
    const EFGModuleBase* operator->() const;  // allow const member fns to be called
  };



  class EFGModuleBase: public ModuleBase
  {
  protected:
//??? default dtor is fine???    virtual ~EFGModuleBase(); // this class is abstract!!

  public:
    typedef ModuleRawPtr RawPtr;
    typedef const ModuleRawPtr& ConstRawPtr;

    virtual std::size_t myNumCompts() const = 0;
    virtual ConstRefRingElem myCompt(const RawPtr& v, size_t pos) const = 0;
    virtual const FreeModule& myAmbientFreeModule() const = 0;
    virtual const std::vector<ModuleElem>& myGens() const = 0;
    virtual const std::vector<ModuleElem>& myTidyGens() const = 0;
  };


  std::size_t NumCompts(const EFGModule& M);
  const std::vector<ModuleElem>& gens(const EFGModule& M);
  const std::vector<ModuleElem>& TidyGens(const EFGModule& M);


  /////////////////////////////////////////////////////////////////////////////
  // Inline functions


  inline bool IsEFGModule(const module& M)
  {
    //    return (dynamic_cast<const EFGModuleBase*>(M.myModulePtr()) != 0);
    return (dynamic_cast<const EFGModuleBase*>(M.myRawPtr()) != 0);
  }


  inline EFGModule AsEFGModule(const module& M)
  {
    if (!IsEFGModule(M))
      CoCoA_ERROR(ERR::NotEFGModule, "AsEFGModule(module)");
    //    return EFGModule(static_cast<const EFGModuleBase*>(M.myModulePtr()));
    return EFGModule(static_cast<const EFGModuleBase*>(M.myRawPtr()));
  }


  inline EFGModule::EFGModule(const EFGModuleBase* MPtr):
      module(MPtr)
  {}


  inline EFGModule::EFGModule(const module& copy):
      module(copy)
  {
    if (!IsEFGModule(copy))
      CoCoA_ERROR("Explicitly finitely generated module required", "EFGModule(module)");
  }


//   inline EFGModule::~EFGModule()
//   {}


  inline const EFGModuleBase* EFGModule::myModulePtr() const
  {
    return static_cast<const EFGModuleBase*>(myRawPtr());
  }


  inline const EFGModuleBase* EFGModule::operator->() const
  {
    return static_cast<const EFGModuleBase*>(myRawPtr());
  }


  inline std::size_t NumCompts(const EFGModule& M)
  {
    return M->myNumCompts();
  }


  inline const FreeModule& AmbientFreeModule(const EFGModule& M)
  {
    return M->myAmbientFreeModule();
  }


  inline const std::vector<ModuleElem>& gens(const EFGModule& M)
  {
    return M->myGens();
  }


  inline const std::vector<ModuleElem>& TidyGens(const EFGModule& M)
  {
    return M->myTidyGens();
  }

} // end of namespace CoCoA



// RCS header/log in the next few lines
// $Header: /Volumes/Home/cocoa/cvs-repository/CoCoALib-0.99/include/CoCoA/EFGModule.H,v 1.1.1.1 2007/03/09 15:16:11 abbott Exp $
// $Log: EFGModule.H,v $
// Revision 1.1.1.1  2007/03/09 15:16:11  abbott
// Imported files
//
// Revision 1.4  2006/12/06 17:11:16  cocoa
// -- removed #include "config.H"
//
// Revision 1.3  2006/11/14 17:46:20  cocoa
// -- changed: reference counting in modules now uses SmartPtrIRC
//
// Revision 1.2  2006/10/06 14:04:16  cocoa
// Corrected position of #ifndef in header files.
// Separated CoCoA_ASSERT into assert.H from config.H;
// many minor consequential changes (have to #include assert.H).
// A little tidying of #include directives (esp. in Max's code).
//
// Revision 1.1.1.1  2006/05/30 11:39:36  cocoa
// Imported files
//
// Revision 1.6  2006/04/21 14:56:33  cocoa
// Changed return type of myCompt member function: now it returns a
// ConstRefRingElem instead of a RingElem (i.e. a copy).
//
// Revision 1.5  2006/03/27 12:21:26  cocoa
// Minor silly changes to reduce number of complaints from some compiler or other.
//
// Revision 1.4  2005/12/31 12:22:18  cocoa
// Several minor tweaks to silence the Microsoft compiler:
//  - added some missing #includes and using directives
//  - moved some function defns into the right namespace
//  - etc.
//
// Revision 1.3  2005/11/29 13:04:47  cocoa
// -- added "const" to myCompt argument
//
// Revision 1.2  2005/11/24 16:09:38  cocoa
// -- added operator[] for ModuleElem
//
// Revision 1.1.1.1  2005/10/17 10:46:54  cocoa
// Imported files
//
// Revision 1.2  2005/09/28 11:50:35  cocoa
// -- new code for graded modules
//
// Revision 1.1.1.1  2005/05/03 15:47:30  cocoa
// Imported files
//
// Revision 1.4  2005/04/20 15:40:48  cocoa
// Major change: modified the standard way errors are to be signalled
// (now via a macro which records filename and line number).  Updated
// documentation in error.txt accordingly.
//
// Improved the documentation in matrix.txt (still more work to be done).
//
// Revision 1.3  2005/04/19 14:06:04  cocoa
// Added GPL and GFDL licence stuff.
//
// Revision 1.2  2005/02/11 14:15:20  cocoa
// New style ring elements and references to ring elements;
// I hope I have finally got it right!
//
// Revision 1.1.1.1  2005/01/27 15:12:13  cocoa
// Imported files
//
// Revision 1.3  2004/11/12 15:49:29  cocoa
// Tidying prior to 0.90 release.
// (a) documentation improved (or marked as poor)
// (b) sundry minor improvements to the code
//
// Revision 1.2  2004/06/29 17:10:22  cocoa
// Partially tidied use of "protected" and "private" in various
// base classes.  Checking in at the end of the day -- it works,
// and I wouldn't want it to be lost next time point's disk
// misbehaves.
//
// Revision 1.1  2004/01/28 15:54:09  cocoa
// Sundry additions.
//
//

#endif
