#ifndef CoCoA_SpecialMatrix_H
#define CoCoA_SpecialMatrix_H

//   Copyright (c)  2005  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License (version 2)
//   as published by the Free Software Foundation.  A copy of the full
//   licence may be found in the file COPYING in this directory.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoA; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include <cstddef>
//using std::size_t;
#include <vector>
// using std::vector;


namespace CoCoA
{

  class ConstMatrix;  // forward declaration -- defined in matrix.H
  class ring;         // forward declaration -- defined in ring.H

  /*-----------------------------------------------------------------*/
  /** \include SpecialMatrix.txt  */
  /*-----------------------------------------------------------------*/

  ConstMatrix NewZeroMatrix(const ring& R, std::size_t NumRows, std::size_t NumCols); ///< Public fn to create a zero matrix
  ConstMatrix NewIdentityMatrix(const ring& R, std::size_t dim);
  ConstMatrix NewTransposeMatrix(ConstMatrix M);
  ConstMatrix NewSubmatrix(ConstMatrix M, const std::vector<std::size_t>& rows, const std::vector<std::size_t>& cols);
  /***************************************************************************/
  // BlockMatrixImpl -- block matrices  ( A  B )
  //                                    ( C  D )
  ConstMatrix NewBlockMatrix(ConstMatrix A, ConstMatrix B, ConstMatrix C, ConstMatrix D);

} // end of namespace CoCoA



// RCS header/log in the next few lines
// $Header: /Volumes/Home/cocoa/cvs-repository/CoCoALib-0.99/include/CoCoA/SpecialMatrix.H,v 1.1.1.1 2007/03/09 15:16:11 abbott Exp $
// $Log: SpecialMatrix.H,v $
// Revision 1.1.1.1  2007/03/09 15:16:11  abbott
// Imported files
//
// Revision 1.3  2006/11/27 16:18:33  cocoa
// -- moved classes declarations from .H to .C (DenseMatrix, DiagMatrix,
//    FieldIdeal, SpecialMatrix)
//
// Revision 1.2  2006/10/06 14:04:15  cocoa
// Corrected position of #ifndef in header files.
// Separated CoCoA_ASSERT into assert.H from config.H;
// many minor consequential changes (have to #include assert.H).
// A little tidying of #include directives (esp. in Max's code).
//
// Revision 1.1.1.1  2006/05/30 11:39:37  cocoa
// Imported files
//
// Revision 1.1.1.1  2005/10/17 10:46:54  cocoa
// Imported files
//
// Revision 1.1.1.1  2005/05/03 15:47:31  cocoa
// Imported files
//
// Revision 1.7  2005/04/19 15:39:55  cocoa
// Matrices now use reference counts.
//
// Revision 1.6  2005/04/19 14:06:04  cocoa
// Added GPL and GFDL licence stuff.
//
// Revision 1.5  2005/03/31 16:59:42  cocoa
// Made special matrix ctors private, so a user has to pass via the
// pseudo-ctors (which do all the arg sanity checking).
//
// Revision 1.4  2005/03/30 17:15:14  cocoa
// Cleaned the SpecialMatrix code; a simple test compiles and
// runs fine.
//

#endif
