//   Copyright (c)  2007  John Abbott and Anna Bigatti

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License (version 2)
//   as published by the Free Software Foundation.  A copy of the full
//   licence may be found in the file COPYING in this directory.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoA; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "CoCoA/ServerOp.H"
#include "CoCoA/error.H"

#include <iostream>
using std::endl;
using std::clog;
#include <map>
using std::map;
// #include <string>
using std::string;


namespace CoCoA
{
  const std::string ServerOpBase::ourVarName4 = "MEMORY.PKG.CoCoA5.Result";


  std::ostream& operator<<(std::ostream& out, const ServerOp& o)
  {
    o->myOutputSelf(out);
    return out;
  }

  // ---- default value for ServerOp ----
  class VoidOperation: public ServerOpBase
  {
  public:
    VoidOperation(): ServerOpBase() {};
    ~VoidOperation() {};
    void myOutputSelf(std::ostream& out) const {out << "VoidOperation";}
    void myReadArgs(std::istream& /*in*/) { CoCoA_ERROR("non-existent operator","VoidOperation::myReadArgs"); }
    void myCompute() { CoCoA_ERROR("non-existent operator","VoidOperation::myCompute"); }
    void myWriteResult(std::ostream& /*out*/) const {}
    void myClear() {}
  };


  ServerOp::ServerOp():
    mySmartPtr(new VoidOperation())
  {}


  bool IsVoidOperation(const ServerOp& o)
  {
    return dynamic_cast<const VoidOperation*>(o.myRawPtr()) != 0;
  }


  //---- registry ------------------------------------------------------
  class registry
  {
  public:
    explicit registry() {}
    ~registry() {}

    friend void RegisterOp(const std::string& s, ServerOp o);
    friend ServerOp& GetOperation(const std::string& s);
    friend void CheckOperationRegistry();
  private:
    std::map<std::string, ServerOp> myOperationMap;
    std::string myMultiplyDefinedString;
  };


  registry& GetReferenceToGlobalRegistry()
  {
    static registry GlobalRegistry;
    return GlobalRegistry;
  }


  ServerOp& GetOperation(const std::string& s)
  {
    return GetReferenceToGlobalRegistry().myOperationMap[s];
  }


  void RegisterOp(const std::string& s, ServerOp o)
  {
//     if (s == "")
//       GetReferenceToGlobalRegistry().myMultiplyDefinedString = "cannot set empty string";
//     else if ( !IsVoidOperation(GetOperation(s)) )
//       GetReferenceToGlobalRegistry().myMultiplyDefinedString = s;
//     else
      GetReferenceToGlobalRegistry().myOperationMap.insert(std::make_pair(s, o));
  }


  void CheckOperationRegistry()
  {
    if (GetReferenceToGlobalRegistry().myMultiplyDefinedString != "")
      CoCoA_ERROR("Multiply defined string: " +
                  GetReferenceToGlobalRegistry().myMultiplyDefinedString,
                  "CheckOperationRegistry");
  }
}

// $Header: /Volumes/Home/cocoa/cvs-repository/CoCoALib-0.99/src/AlgebraicCore/ServerOp.C,v 1.1 2007/04/27 14:54:22 bigatti Exp $
// $Log: ServerOp.C,v $
// Revision 1.1  2007/04/27 14:54:22  bigatti
// -- content of CoCoAServer.C split into dedicated files
// -- new registration mechanism (through include "RegisterServerOps.H")
//
