//   Copyright (c)  2005  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License (version 2)
//   as published by the Free Software Foundation.  A copy of the full
//   licence may be found in the file COPYING in this directory.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoA; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include "CoCoA/time.H"

#include <sys/time.h>
#include <sys/resource.h>
#include <unistd.h>


namespace CoCoA
{

  double CpuTime()
  {
    static struct rusage used;

    getrusage(RUSAGE_SELF, &used);
    // Use volatile to force truncation to IEEE doubles in x86 processors;
    // otherwise it is possible to get negative time differences!
    volatile double seconds = used.ru_utime.tv_sec + used.ru_stime.tv_sec +
      (used.ru_utime.tv_usec + used.ru_stime.tv_usec) / 1.0e6;
    return seconds;
  }

  double RealTime()
  {
    static struct timeval tv;
    gettimeofday(&tv, 0); // 0 is a NULL pointer
    // Use volatile to force truncation to IEEE doubles in x86 processors;
    // otherwise it is possible to get negative time differences!
    volatile double seconds = tv.tv_sec + tv.tv_usec/1.0e6;
    return seconds;
  }

} // end of namespace CoCoA


// RCS header/log in the next few lines
// $Header: /Volumes/Home/cocoa/cvs-repository/CoCoALib-0.99/src/AlgebraicCore/time.C,v 1.1.1.1 2007/03/09 15:16:11 abbott Exp $
// $Log: time.C,v $
// Revision 1.1.1.1  2007/03/09 15:16:11  abbott
// Imported files
//
// Revision 1.1.1.1  2006/05/30 11:39:37  cocoa
// Imported files
//
// Revision 1.1  2005/11/15 12:24:35  cocoa
// New timing functions.
//
