#include "CoCoA/BuildInfo.H"
#include "CoCoA/FractionField.H"
#include "CoCoA/FreeModule.H"
#include "CoCoA/GlobalManager.H"
#include "CoCoA/RingDistrMPoly.H"
#include "CoCoA/RingQ.H"
#include "CoCoA/SparsePolyRing.H"
#include "CoCoA/degree.H"
#include "CoCoA/io.H"

using namespace CoCoA;

#include <iostream>
using std::cout;
using std::endl;
#include <vector>
using std::vector;

//----------------------------------------------------------------------
// Test for GradedFreeModules
// functions:  LPos(v),  LPP(v),  wdeg(v),  IsHomogeneous(v)
// environments: Q[x,y,z]
//----------------------------------------------------------------------

#define TEST_ASSERT(cond)   do { if (!(cond)) CoCoA::AssertionFailed(#cond, __FILE__, __LINE__); } while(0)


// This trial is still very rudimentary..

void trial(ring R)
{
  const size_t dim = 4;
  const FreeModule F = NewFreeModule(R, dim);
  const vector<ModuleElem>& e = gens(F);
  const vector<RingElem>& x = indets(AsSparsePolyRing(R));
  TEST_ASSERT(e.size() == dim);
// This does not yet compile
//   for (size_t i=0; i < dim; ++i)
//     for (size_t j=0; j < dim; ++j)
//       TEST_ASSERT(Fgens[i][j] == (i==j));
  TEST_ASSERT(IsEFGModule(F));


  ModuleElem v1 = (x[0]-x[1])*e[0] + (x[0]*x[0]-x[1]*x[2])*e[1]
    + x[0]*x[0]*e[2] + (x[1]*x[2])*e[3];
  ModuleElem v2 = 4*e[0] + 3*e[1] + 2*e[2] + 1*e[3];
  GlobalOutput() << "v1 = " << v1 << std::endl;
  GlobalOutput() << "LPos(v1) = " << LPos(v1) << std::endl;
  GlobalOutput() << "LPP(v1) = " << LPP(v1) << std::endl;
  GlobalOutput() << "wdeg(v1) = " << wdeg(v1) << std::endl;
  GlobalOutput() << "IsHomogeneous(v1) = " << IsHomogeneous(v1) << std::endl;
  GlobalOutput() << "IsHomogeneous(v2) = " << IsHomogeneous(v2) << std::endl;

  //  CmpWDeg(v1, v2);
  //  CmpLT(v1, v2);
  //  CmpPos(v1, v2);
}


void program()
{
  GlobalManager CoCoAFoundations;

  const ring P = NewPolyRing(RingQ(), 3);

  //  trial(Z);
  trial(P);
}


// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT CoCoA Error";
    ANNOUNCE(err);
  }
  catch (const std::exception& exc)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }

  BuildInfo::PrintAll(GlobalErrput());
  return 1;
}
