#include "CoCoA/BuildInfo.H"
#include "CoCoA/FractionField.H"
#include "CoCoA/GlobalManager.H"
#include "CoCoA/PolyRing.H"
#include "CoCoA/RingDistrMPoly.H"
#include "CoCoA/RingHom.H"
#include "CoCoA/RingQ.H"
#include "CoCoA/SparsePolyRing.H"
#include "CoCoA/io.H"
#include "CoCoA/ring.H"

using namespace CoCoA;

#include <iostream>
using std::cout;
using std::endl;
#include <vector>
using std::vector;

//----------------------------------------------------------------------
// Test for RingHom
// Qxy->Qxy
//----------------------------------------------------------------------

#define TEST_ASSERT(cond)   do { if (!(cond)) CoCoA::AssertionFailed(#cond, __FILE__, __LINE__); } while(0)

void program()
{
  GlobalManager CoCoAFoundations;

  const PolyRing Qxy = NewPolyRing(RingQ(), 2);

  const vector<RingElem>& x = indets(Qxy);
  vector<RingElem> imx;
  imx.push_back(x[0]*x[0]);
  imx.push_back(x[1]*x[1]);
  RingHom Qxy2Qxy = PolyRingHom(Qxy, Qxy, CoeffEmbeddingHom(Qxy), imx);
  RingHom Qxy2QxyBis = PolyAlgebraHom(Qxy, Qxy, imx);

  cout << "Simple test on polynomial ring hom from " << Qxy << "  to  " << Qxy << endl
       << "sending " << indet(Qxy,0) << "  to  " << imx[0] << "  and" << endl
       << "sending " << indet(Qxy,1) << "  to  " << imx[1] << endl;

  RingElem f = x[0]+2*x[1];

  TEST_ASSERT(Qxy2QxyBis(f) == Qxy2Qxy(f));

  cout << f << " in " << owner(f) << "  maps to " << Qxy2Qxy(f) << endl;
}


// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT CoCoA Error";
    ANNOUNCE(err);
  }
  catch (const std::exception& exc)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }

  BuildInfo::PrintAll(GlobalErrput());
  return 1;
}
