#include "CoCoA/BuildInfo.H"
#include "CoCoA/GlobalManager.H"
#include "CoCoA/FractionField.H"
#include "CoCoA/RingDistrMPoly.H"
#include "CoCoA/TmpIsTree.H"
#include "CoCoA/RingFp.H"
#include "CoCoA/RingQ.H"
#include "CoCoA/SparsePolyRing.H"
#include "CoCoA/io.H"
#include "CoCoA/symbol.H"

using namespace CoCoA;

#include <iostream>
using std::endl;
#include <vector>

//----------------------------------------------------------------------
// Test for TmpIsTree
//----------------------------------------------------------------------

#define TEST_ASSERT(cond)   do { if (!(cond)) CoCoA::AssertionFailed(#cond, __FILE__, __LINE__); } while(0)


void TestCycle()
{
  SparsePolyRing P(NewPolyRing(RingQ(),6));
  RingElem x = indet(P,0),  y = indet(P,1),  z = indet(P,2);
  RingElem t = indet(P,3),  u = indet(P,4),  v = indet(P,5);

  PolyList PL;
  PL.push_back(x*y);
  PL.push_back(y*z);
  PL.push_back(z*t);
  PL.push_back(t*u);
  PL.push_back(u*v);
  PL.push_back(v*x);
  FacetComplex Cycle(P,PL);
  std::list<facet> Result=Cycle.myIsTreeNoOpt();
  GlobalOutput() << "myIsTreeNoOpt: Cycle in Complex = " << Result << endl;
  Result=Cycle.myIsTreeNoOpt();
  GlobalOutput() << "myIsTreeNoOpt: Cycle in Complex = " << Result << endl;
  Result=Cycle.myIsTreeCBNoOpt();
  GlobalOutput() << "myIsTreeCBNoOpt: Cycle in Complex = " << Result << endl;
  Result=Cycle.myIsTreeCBOpt();
  GlobalOutput() << "myIsTreeCBOpt: Cycle in Complex = " << Result << endl;
}

void TestLine()
{
  SparsePolyRing P(NewPolyRing(RingQ(),6));
  RingElem x = indet(P,0),  y = indet(P,1),  z = indet(P,2);
  RingElem t = indet(P,3),  u = indet(P,4),  v = indet(P,5);

  PolyList PL;
  PL.push_back(x*y);
  PL.push_back(y*z);
  PL.push_back(z*t);
  PL.push_back(t*u);
  PL.push_back(u*v);
  FacetComplex Cycle(P,PL);
  std::list<facet> Result=Cycle.myIsTreeNoOpt();
  GlobalOutput() << "myIsTreeNoOpt: Cycle in Complex = " << Result << endl;
  Result=Cycle.myIsTreeNoOpt();
  GlobalOutput() << "myIsTreeNoOpt: Cycle in Complex = " << Result << endl;
  Result=Cycle.myIsTreeCBNoOpt();
  GlobalOutput() << "myIsTreeCBNoOpt: Cycle in Complex = " << Result << endl;
  Result=Cycle.myIsTreeCBOpt();
  GlobalOutput() << "myIsTreeCBOpt: Cycle in Complex = " << Result << endl;
}


void program()
{
  GlobalManager CoCoAFoundations;
  
  TestCycle();
  TestLine();
}


// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT CoCoA Error";
    ANNOUNCE(err);
  }
  catch (const std::exception& exc)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }

  BuildInfo::PrintAll(GlobalErrput());
  return 1;
}
