// This bug originally reported by Dag Arneson 2005/04/01
// It used to provoke SEGV.

#include "CoCoA/BuildInfo.H"
#include "CoCoA/FractionField.H"
#include "CoCoA/GlobalManager.H"
#include "CoCoA/PPMonoidEv.H"
#include "CoCoA/PPOrdering.H"
#include "CoCoA/QuotientRing.H"
#include "CoCoA/RingDistrMPoly.H"
#include "CoCoA/RingQ.H"
#include "CoCoA/SparsePolyRing.H"
#include "CoCoA/io.H"
#include "CoCoA/symbol.H"

using namespace CoCoA;

// Includes from the standard C++ library
#include <iostream>
using std::endl;
#include <cstddef>
using std::size_t;

void program()
{
  GlobalManager CoCoAFoundations;

  //To make the polynomial ring we have to make a PP monoid
  PolyRing Qp = NewPolyRing(RingQ(), SymbolRange("y", 0, 2)); // Q[y1..yn]

  RingElem foo(Qp);
  foo = 5*indet(Qp,0)*indet(Qp,1)+2*indet(Qp,0)*indet(Qp,2);
  ideal I(Qp, foo);
  QuotientRing R = NewQuotientRing(Qp,I);
}


//----------------------------------------------------------------------
// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT CoCoA Error";
    ANNOUNCE(err);
  }
  catch (const std::exception& exc)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }

  BuildInfo::PrintAll(GlobalErrput());
  return 1;
}
