#include "CoCoA/BuildInfo.H"
#include "CoCoA/DenseMatrix.H"
#include "CoCoA/FractionField.H"
#include "CoCoA/GlobalManager.H"
#include "CoCoA/RingQ.H"
#include "CoCoA/RingZ.H"
#include "CoCoA/io.H"
#include "CoCoA/matrix.H"

using namespace CoCoA;
#include <iostream>
using std::endl;

// convention: a function containing a "new" should be named "New.."
matrix NewMatrixFromC(ring K, int cmat[4][4])
{
  size_t NumRows = 4;
  size_t NumCols = 4;
  matrix M(NewDenseMatrix(K,NumRows,NumCols));

  for (size_t i=0; i < NumRows; ++i)
    for (size_t j=0; j < NumCols; ++j)
      SetEntry(M, i, j, cmat[i][j]);
  return M;
}


void TestMatrix(matrix M)
{
  GlobalOutput() << M << endl;
  if (NumRows(M)==NumCols(M))
    GlobalOutput() << "det(M) gives " << det(M) << endl;
  GlobalOutput() << "rank(M) gives " << rank(M) << endl;
  GlobalOutput() << "---------------------------------------------" << endl;
}


void program()
{
  GlobalManager CoCoAFoundations;

  const ring Z = RingZ();
  const ring Q = RingQ();

  int M1[4][4] = {{1, 0, 0, 4},
                  {0, 3, 2, 0},
                  {0, 3, 2, 1},
                  {1, 0, 0, 0}};
  TestMatrix(NewMatrixFromC(Z, M1));
  TestMatrix(NewMatrixFromC(Q, M1));

  int M2[4][4] = {{1, 0, 0, 0},
                  {0, 3, 2, 0},
                  {0, 3, 2, 1},
                  {1, 1, 1, 0}};
  TestMatrix(NewMatrixFromC(Z, M2));
  TestMatrix(NewMatrixFromC(Q, M2));
}


// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT CoCoA Error";
    ANNOUNCE(err);
  }
  catch (const std::exception& exc)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }

  BuildInfo::PrintAll(GlobalErrput());
  return 1;
}
